local L = LibStub("AceLocale-3.0"):GetLocale("HiveMind")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local extractSpellID

local options = {
	type = "group",
	args = {
		config = {
			type = "execute",
			guiHidden = true,
			name = L["Configure"],
			desc = L["Configure"],
			func = HiveMind.OpenConfig
		},
		showSelf = {
			type = "toggle",
			name = L["Show Self"],
			desc = L["Show Self"],
			get = function(v)
				return HiveMind.db.profile.showSelf
			end,
			set = function(info, v)
				HiveMind.db.profile.showSelf = v
				HiveMind:LayoutMembers()
			end
		},
		showInArenas = {
			type = "toggle",
			name = L["Only Show In Arena"],
			desc = L["Only Show In Arena"],
			get = function(v)
				return HiveMind.db.profile.showOnlyInArena
			end,
			set = function(info, v)
				HiveMind.db.profile.showOnlyInArena = v
				HiveMind:ZONE_CHANGED_NEW_AREA()
			end
		},
		scale = {
			type = "range",
			name = L["Scale"],
			desc = L["Scale"],
			min = 0.2,
			max = 5.0,
			step = 0.2,
			bigStep = 0.2,
			get = function(v)
				return HiveMind.db.profile.scale
			end,
			set = function(info, v)
				HiveMind.db.profile.scale = v
				HiveMind.anchor:SetScale(v)
			end
		},
		lock = {
			type = "toggle",
			name = L["Lock"],
			desc = L["Lock"],
			get = function(v)
				return HiveMind.db.profile.locked
			end,
			set = function(info, v)
				HiveMind.db.profile.locked = v
			end
		},
		--[[
		spells = {
			type = "group",
			name = L["Spells"],
			args = {}
		},
		items = {
			type = "group",
			name = L["Items"],
			args = {}
		},		
		]]--
		addSpell = {
			type = "input",
			name = L["Add Spell"],
			desc = L["Add Spell"],
			get = function() end,
			set = function(info, v)
				local id = extractSpellID(v)
				tinsert(HiveMind.db.profile.spells, v)
				local n = UnitClass("player")
				HiveMind:InitAbilities(HiveMind.db.profile.spells, n)
			end
		},
		addItem = {
			type = "input",
			name = L["Add Item"],
			desc = L["Add Item"],
			get = function() end,
			set = function(info, v)
				local id = extractItemID(v)
				tinsert(HiveMind.db.profile.items, v)
				HiveMind:InitItems(HiveMind.db.profile.items)
			end
		}		
	}
}

HiveMind.options = options

function HiveMind:AddSpellOptionsTable(key, src, dst)
	dst[key] = dst[key] or {
		type = "group",
		name = key,
		args = {}
	}
	
	for _, v in pairs(src) do
		local name = GetSpellInfo(v)
		DEFAULT_CHAT_FRAME:AddMessage("Getting for " .. tostring(v) .. ": " .. tostring(name))
		dst[key].args[name:gsub(" ", "")] = {
			type = "toggle",
			name = name,
			desc = name,
			get = function() return not self.db.profile.hiddenSpells[v] end,
			set = function(info, b)
				self.db.profile.hiddenSpells[v] = not b
				self:LayoutMembers()
			end,
		}
	end
end

function HiveMind:AddItemOptionsTable(src, dst)
	for _, v in pairs(src) do
		local name = GetItemInfo(v)
		dst[name:gsub(" ", "")] = dst[name] or {
			type = "toggle",
			name = name,
			desc = name,
			get = function() return not self.db.profile.hiddenItems[v] end,
			set = function(info, b)
				self.db.profile.hiddenItems[v] = not b
				self:LayoutMembers()
			end,
		}
	end
end

AceConfig:RegisterOptionsTable("HiveMind", options)

function extractSpellID(name)
	local id = GetSpellLink(name):match("Hspell:(%d+)")
	if id then
		return tonumber(id)
	else
		HiveMind:Print(("Unable to get spell ID for %s. Check your spelling."):format(name))
	end
end

function extractItemID(name)
	local id = select(2, GetItemInfo(name)):match("Hitem:(%d+)")
	if id then
		return tonumber(id)
	else
		HiveMind:Print(("Unable to get item ID for %s. Check your spelling."):format(name))
	end
end
