HiveMind = LibStub("AceAddon-3.0"):NewAddon("HiveMind", "AceConsole-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceEvent-3.0", "AceTimer-3.0")

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("HiveMind", false)

--[[
-- For autolocalization
L["Druid"]
L["Mage"]
L["Hunter"]
L["Priest"]
L["Paladin"]
L["Rogue"]
L["Shaman"]
L["Warrior"]
L["Warlock"]
]]--

local classKeys = {"ROGUE", "MAGE", "PRIEST", "PALADIN", "DRUID", "HUNTER", "SHAMAN", "WARRIOR", "WARLOCK"}
local cooldowns

local playerClass
local abilities, items = {}, {}
HiveMind.CombatLog = {}
HiveMind.Comm = {}
local PREFIX = "HIVE"
local optFrame

local defaults = {
	profile = {
		showSelf = true,
		scale = 1,
		hiddenSpells = {},
		hiddenItems = {},
		spells = {},
		items = {},
		locked = false
	}
}

function HiveMind:OnInitialize()
	cooldowns = self.cooldowns
	-- self.cooldowns = nil
	self.db = LibStub("AceDB-3.0"):New("HiveMindDB", defaults)
	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self:RegisterComm(PREFIX)
	self:CreateAnchor()
	optFrame = AceConfigDialog:AddToBlizOptions("HiveMind", "HiveMind")
end

function HiveMind:OnEnable()
	_, playerClass = UnitClass("player")
	self:RegisterEvent("ARENA_TEAM_ROSTER_UPDATE")
	
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PARTY_MEMBER_DISABLE", "LayoutMembers")
	self:RegisterEvent("PARTY_MEMBER_ENABLE", "LayoutMembers")
	
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:ZONE_CHANGED_NEW_AREA()
	self:EnterArena()
	self:LayoutMembers()
	
	self:InitItems(cooldowns.ITEMS)
	self:SendItems()

end

function HiveMind:RegisterEvents()
	if self.registered then return end
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self.registered = true
end

function HiveMind:UnregisterEvents()
	self.registered = false
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
	self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
	self:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
end

function HiveMind:PARTY_MEMBERS_CHANGED()
	if self.numPartyMembers ~= GetNumPartyMembers() then
		self:ZONE_CHANGED_NEW_AREA()
		self:EnterArena()
		self:LayoutMembers()
		self.numPartyMembers = GetNumPartyMembers()
		
		self:InitItems(cooldowns.ITEMS)
		self:SendItems()
	end
end

function HiveMind:ZONE_CHANGED_NEW_AREA()
	if not self.db.profile.showOnlyInArena or select(2, IsInInstance()) == "arena" then
		self:RegisterEvents()
		self.anchor:Show()
	else
		self:UnregisterEvents()
		self.anchor:Hide()
	end
end

function HiveMind:ARENA_TEAM_ROSTER_UPDATE()
	local inInstance, instanceType = IsInInstance()
	if inInstance and instanceType == "arena" then
		self:EnterArena()
	end
end

function HiveMind:InitAbilities(list)
	if not list then return end
	for k, v in ipairs(list) do
		local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(v)
		
		-- Translate the highest rank into our max rank
		name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(name)
		if name then
			local spellID = GetSpellLink(name):match("Hspell:(%d+)")
			abilities[tonumber(spellID)] = icon
		end
	end
end

function HiveMind:InitItems(list)
	if not list then return end
	for k, v in pairs(items) do
		items[k] = nil
	end
	for k, v in ipairs(list) do
		if IsEquippedItem(v) then
			local name, link, rarity, _, _, _, _, _, _, tex = GetItemInfo(v)
			if not name then
				GameTooltip:SetHyperlink("item:" .. v)
				name, link, rarity, _, _, _, _, _, _, tex = GetItemInfo(v)
			end
			if name then
				items[v] = tex
			end
		end
	end
end

function HiveMind:UNIT_INVENTORY_CHANGED()
	self:InitItems(cooldowns.ITEMS)
	self:SendItems()
end

function HiveMind:UNIT_SPELLCAST_SUCCEEDED(evt, arg1, arg2)
	if arg1 == "player" and arg2 == L["PvP Trinket"] then		
		self:ScheduleTimer("SendItems", 0.5)
	end
end

function HiveMind:EnterArena()
	self:InitAbilities(cooldowns[playerClass])
	self:InitAbilities(cooldowns.RACIALS)
	self:InitAbilities(self.db.profile.spells)
	
	self:InitItems(cooldowns.ITEMS)
	self:InitItems(self.db.profile.items)
	
	self:SendAbilities()
	self:SendItems()
end

function HiveMind:SendAbilities()
	for k, v in pairs(abilities) do
		self:SendSpellUpdate(k, self:CDInfo(k))
	end
end

function HiveMind:SendItems()
	for k, v in pairs(items) do
		self:SendItemUpdate(k, self:CDInfo(k, true))
	end
end

function HiveMind:SendSpellUpdate(spellID, cdRemaining)
	local data = self:Serialize("ABILITY", spellID, cdRemaining)
	self.Comm.ABILITY(self, UnitName("player"), spellID, cdRemaining)
	self:SendCommMessage(PREFIX, data, "PARTY")
end

function HiveMind:SendItemUpdate(itemID, cdRemaining)
	local data = self:Serialize("ITEM", itemID, cdRemaining)
	self.Comm.ITEM(self, UnitName("player"), itemID, cdRemaining)
	self:SendCommMessage(PREFIX, data, "PARTY")
end

function HiveMind:CDInfo(k, isItem)
	local a, b, c
	if isItem then
		a, b, c = GetItemCooldown(k)
	else
		a, b, c = GetSpellCooldown(GetSpellInfo(k))	
	end
	if not a or (a == 0) then return 0 end	
	return math.ceil(b - (GetTime() - a))
end

--[[ Combat Log ]]--

function HiveMind:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags = ...
	local handler = self.CombatLog[eventtype]
	if handler then
		handler(self, ...)
	end
end

function HiveMind:SPELL_UPDATE_COOLDOWN()
	for k, v in pairs(abilities) do
		local t = self:CDInfo(k)
		
		-- ignore short cooldowns caused by interruption or some GC
		-- to minimize traffic
		if t > 14 then
			self:SendSpellUpdate(k, t)
		end

	end
end

--[[ Comms ]]--

function HiveMind:OnCommReceived(prefix, message, distribution, sender)
	local success, a, b, c, d, e, f, g, h, i, j, k = self:Deserialize(message)
	if sender == UnitName("player") then return end
	if self.Comm[a] then
		self.Comm[a](self, sender, b, c, d, e, f, g, h, i, j, k)
	end
end

function HiveMind.Comm:ABILITY(sender, spellID, remaining)
	if self:CreateMemberAnchor(sender) then
		self:LayoutMembers()
	end
	if self:SetButton(sender, spellID, remaining) then
		self:LayoutButtons(sender)
	end
end

function HiveMind.Comm:ITEM(sender, itemID, remaining)
	if self:CreateMemberAnchor(sender) then
		self:LayoutMembers()
	end
	if self:SetButton(sender, itemID, remaining, true) then
		self:LayoutButtons(sender)
	end
end
