local strutf8sub = _G.string.utf8sub
local strgmatch	= _G.string.gmatch

local buttons = {}
local buttonPrototype = {}
local buttonPrototype_mt = {__index = buttonPrototype}
local frame_defaults = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	inset = 4,
	edgeSize = 8,
	tile = true,
	insets = {left = 2, right = 2, top = 2, bottom = 2}
}

local function mousedown(self)
	if not HiveMind.db.profile.locked then
		HiveMind.anchor:StartMoving()
	end
end
local function mouseup(self) HiveMind.anchor:StopMovingOrSizing() end
local function enterFrame(self)
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	if self.isItem then
		GameTooltip:SetHyperlink("item:" .. self.spellID)
	else
		GameTooltip:SetHyperlink("spell:" .. self.spellID)
	end
	GameTooltip:Show();
end

local dropdown = CreateFrame("Frame", "HiveMenuDropdown", UIParent, "UIDropDownMenuTemplate")

local menu = {
	{
		text = "Hide this",
		func = function(self)
			self.owner.hidden = true
			self:Hide();
			HiveMind:LayoutMembers()
		end
	}
}
local function click(self, button)
	if button == "RightButton" then
		menu[1].text = "Hide " .. GetSpellInfo(self.spellID)
		menu[1].arg1 = self
		EasyMenu(menu, dropdown, "cursor", 0, 0, "MENU")
	end
end

local function leaveFrame(self) GameTooltip:Hide() end
local GetFormattedTime, ShortenName

function buttonPrototype:Create(who, name, icon, remaining, spellID, isItem)
	local frame = CreateFrame("Button", "HiveMind-" .. who .. "-" .. name, HiveMind.members[who])
	frame:SetWidth(30)
	frame:SetHeight(30)
	frame.name = who
	self.frame = frame
	frame.owner = self
	-- frame:SetBackdrop(frame_defaults)
	frame:EnableMouse(true)
	frame:SetScript("OnMouseDown", mousedown)
	frame:SetScript("OnMouseUp", mouseup)
	frame:SetScript("OnEnter", enterFrame)
	frame:SetScript("OnLeave", leaveFrame)
	frame:SetScript("OnClick", click)
	frame:RegisterForClicks("RightButtonUp")
	frame.isItem = isItem
	frame.spellID = spellID
	
	local tex = frame:CreateTexture()
	tex:SetAllPoints()
	tex:SetTexture(icon)
	self.tex = tex;
	
	local label = frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	label:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT")
	label:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
	label:SetText(ShortenName(name))
	local f, h, l = label:GetFont()
	label:SetFont(f, 8, l)
	self.label = label
	
	local timeLeft = frame:CreateFontString(nil, nil, "GameFontNormal")
	timeLeft:SetPoint("CENTER", frame, "CENTER")
	local f, h, l = timeLeft:GetFont()
	timeLeft:SetFont(f, 14, l)
	self.timeLeft = timeLeft
	
	self:SetTimeLeft(remaining)
	return self
end

do
	local updateTime = 0
	local function update(self, arg1)
		self.updateTime = self.updateTime or 0
		self.updateTime = self.updateTime + arg1
		if self.updateTime > 0.25 then
			self.updateTime = self.updateTime - 0.25
			self.owner:SetTimeLeft(self.owner.targetTime - GetTime())
		end
	end

	function buttonPrototype:SetTimeLeft(t)
		if (t <= 0) then
			self.timeLeft:Hide()
			self.label:SetTextColor(1, 1, 1)
			self.frame:SetScript("OnUpdate", nil)
			self.tex:SetVertexColor(1, 1, 1)
			self.tex:SetAlpha(1)
		else
			self.label:SetTextColor(0.6, 0.6, 0.6)
			self.timeLeft:SetText(GetFormattedTime(t))
			self.timeLeft:Show()
			self.initTime = GetTime()
			self.remaining = t
			self.targetTime = GetTime() + t
			self.frame:SetScript("OnUpdate", update)
			self.tex:SetVertexColor(0.2, 0.2, 0.2)
			self.tex:SetAlpha(0.8)
			-- self.tex:SetDesaturated(true)
		end
	end
end

function HiveMind:CreateButton(who, spellID, remaining, isItem)
	local name, icon
	if isItem then
		name, _, _, _, _, _, _, _, _, icon = GetItemInfo(spellID)
	else
		name, _, icon = GetSpellInfo(spellID)
	end
	if not name then
		DEFAULT_CHAT_FRAME:AddMessage("Can't add spell for " .. spellID)
		return
	end
	buttons[who] = buttons[who] or {}
	if buttons[who][name] then return buttons[who][name], false end
	buttons[who][name] = setmetatable({}, buttonPrototype_mt)
	return buttons[who][name]:Create(who, name, icon, remaining, spellID, isItem), true
end

function HiveMind:SetButton(who, spellID, remaining, isItem)
	local button, status = self:CreateButton(who, spellID, remaining, isItem)
	button:SetTimeLeft(remaining)
	return status
end

function HiveMind:CreateMemberAnchor(who)
	self.members = self.members or {}
	if self.members[who] then return nil end
	self.members[who] = CreateFrame("Frame", "HiveMind-" .. who, self.anchor)
	local frame = self.members[who]
	-- frame:SetBackdrop(frame_defaults)
	local label = frame:CreateFontString(nil, nil, "GameFontNormal")
	label:SetPoint("BOTTOMLEFT", frame, "TOPLEFT")
	label:SetText(who)
	frame:SetHeight(30)
	return self.members[who]
end

function HiveMind:CreateAnchor()
	self.anchor = CreateFrame("Frame", "HiveMindAnchor", UIParent)
	self.anchor:SetScale(self.db.profile.scale)
	self.anchor:SetPoint("CENTER", UIParent, "CENTER")
	self.anchor:SetWidth(10)
	self.anchor:SetHeight(10)	
	self.anchor:SetMovable(true)
end

do
	local keys = {}
	function HiveMind:LayoutMembers()
		for i = 1, #keys do tremove(keys) end
		if not self.members then return end
		
		for k, v in pairs(self.members) do tinsert(keys, k) end
		table.sort(keys)
		
		for k, v in pairs(self.members) do v:ClearAllPoints() end
		local last = nil
		for i = 1, #keys do
			if UnitInParty(keys[i]) and not UnitIsDead(keys[i]) and (keys[i] ~= UnitName("player") or HiveMind.db.profile.showSelf) then
				self.members[keys[i]]:Show()
				local f = self.members[keys[i]]
				if last then
					f:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -25)
				else
					f:SetPoint("TOPLEFT", self.anchor, "TOPLEFT")
				end
				last = f
				self:LayoutButtons(keys[i])
			else
				self.members[keys[i]]:Hide()
			end
		end
	end
end

do
	local keys = {}
	function HiveMind:LayoutButtons(who)
		if not buttons[who] then return end
		for i = 1, #keys do tremove(keys) end
		for k, v in pairs(buttons[who]) do
			local hide = (not v.isItem and self.db.profile.hiddenSpells[v.spellID]) or (v.isItem and self.db.profile.hiddenItems[v.spellID])
			if not v.hidden and not hide then
				tinsert(keys, k)
			else
				v.frame:Hide()
			end
		end		
		table.sort(keys)
		
		for k, v in pairs(buttons[who]) do
			v.frame:ClearAllPoints()
		end
		local last = nil
		for i = 1, #keys do
			local f = buttons[who][keys[i]].frame
			if last then
				f:SetPoint("TOPLEFT", last, "TOPRIGHT", 2, 0)
			else
				f:SetPoint("TOPLEFT", self.members[who], "TOPLEFT")
			end
			last = f
		end
		self.members[who]:SetWidth(#keys * (last:GetWidth() + 2))
	end
end

-- Stoled from CooldownCount
function GetFormattedTime(secs)
	-- return cc now value, next update value, font size, toggle two color
	if secs >= 86400 then	
		return ceil(secs / 86400) .. "d", mod(secs, 86400), 18
	elseif secs >= 3600 then
		return ceil(secs / 3600) .. "h", mod(secs, 3600), 18
	elseif secs >= 600 then
		return ceil(secs / 60) .. "m", mod(secs, 60), 18
	elseif secs >= 60 then
		return ceil(secs / 60) .. "m", mod(secs, 60), 24
	elseif secs >= 10 then
		return floor(secs + 0.5), secs - floor(secs), 28
	end
	return floor(secs + 0.5), 0.5, 34, true
end

-- From EBB
function ShortenName(name)
	if #name <= 6 then return name end
	local shortname = ""
	if strmatch(name, "[%(%)]") then
		name = strtrim(strgsub(strgsub(name, "%b()", ""), "  +", " "))
	end
	local ct = 0
	for word in strgmatch(name, "([^ ]+)") do
		ct = ct + 1
		local alpha, bravo = strutf8sub(word, 1, 1), strutf8sub(word, 2)
		if strmatch(bravo, ":") then
			shortname = shortname..alpha .. ":"
		else
			shortname = shortname..alpha
		end
	end
	if ct == 1 then
		return name:sub(0, 6)
	end 
	return shortname
end
